﻿    using System;
    using System.Collections.Generic;
    using System.Text;

    namespace reversi
    {
    class Movement
    {
    public  Board board;
    public Movement(Board Getboard){
        board=new Board(Getboard);
    } 
    //
    // Sets a board with the initial game set-up.
    //
    public void SetForNewGame()
    {
        // Clear the board.
        int i, j;
        for (i = 0; i < 8; i++)
            for (j = 0; j < 8; j++)
            {
               board.squares[i, j] = Board.Empty;
               board.safeDiscs[i, j] = false;
            }

        // Set two black and two white discs in the center.
        board.squares[3, 3] = Board.White;
        board.squares[3, 4] = Board.Black;
        board.squares[4, 3] = Board.Black;
        board.squares[4, 4] = Board.White;

        // Update the counts.
        this.UpdateCounts();
    }
    //
        // Given a player move and a specific direction, determines if any
        // opponent discs will be outflanked.
        // Note: For performance reasons the direction values are NOT checked
        // for validity (dr and dc may be one of -1, 0 or 1 but both should
        // not be zero).
        //
    public bool IsOutflanking(int color, int row, int col, int dr, int dc)
        {

        // Move in the given direction as long as we stay on the board and
        // land on a disc of the opposite color.
        int r = row + dr;
        int c = col + dc;
        while (r >= 0 && r < 8 && c >= 0 && c < 8 && board.squares[r, c] == -color)
        {
             
            r += dr;
            c += dc;
        }

        // If we ran off the board, only moved one space or didn't land on
        // a disc of the same color, return false.
        if (r < 0 || r > 7 || c < 0 || c > 7 || (r - dr == row && c - dc == col) || board.squares[r, c] != color)
            return false;

        // Otherwise, return true;
        return true;
        }
    //
    // Determines if a specific move is valid for the player.
    //
    public bool IsValidMove(int color, int row, int col)
    {
        // The square must be empty.
        if (board.squares[row, col] != Board.Empty)
            return false;

        // Must be able to flip at least one opponent disc.
        int dr, dc;
        for (dr = -1; dr <= 1; dr++)
            for (dc = -1; dc <= 1; dc++)
                if (!(dr == 0 && dc == 0) && IsOutflanking(color, row, col, dr, dc))
                    return true;

        // No opponents could be flipped.
        return false;
    }
    // Defines the game states.
    public void UpdateCounts()

        {
        int colorSquare = 0;
        // Reset all counts.
        board.BlackCount = 0;
        board.WhiteCount = 0;
        board.EmptyCount = 0;
        board.BlackFrontierCount = 0;
        board.WhiteFrontierCount = 0;
        board.WhiteSafeCount = 0;
        board.BlackSafeCount = 0;

        int i, j;

        // Update the safe disc map.
        //
        // All currently unsafe discs are checked to see if they are still
        // outflankable. Those that are not are marked as safe.
        // If any new safe discs were found, the process is repeated
        // because board change may have made other discs safe as well. The
        // loop exits when no new safe discs are found.
        bool statusChanged = true;
        while (statusChanged)
        {
            statusChanged = false;
            for (i = 0; i < 8; i++)
                for (j = 0; j < 8; j++)
                {
                    colorSquare = this.board.squares[i, j];

                    if (this.board.squares[i, j] != Board.Empty && !this.board.safeDiscs[i, j] && !IsValidMove(colorSquare,i, j))
                    {
                        this.board.safeDiscs[i, j] = true;
                        statusChanged = true;
                    }
                 }
        }

        // Tally the counts.
        int dr, dc;
        for (i = 0; i < 8; i++)
            for (j = 0; j < 8; j++)
            {

                // If there is a disc at board position, determine if it is
                // on the frontier (i.e., adjacent to an empty square).
                bool isFrontier = false;
                if (this.board.squares[i, j] != Board.Empty)
                {
                    for (dr = -1; dr <= 1; dr++)
                        for (dc = -1; dc <= 1; dc++)
                            if (!(dr == 0 && dc == 0) && i + dr >= 0 && i + dr < 8 && j + dc >= 0 && j + dc < 8 && board.squares[i + dr, j + dc] == Board.Empty)
                                isFrontier = true;
                }


                // Update the counts.
                if (this.board.squares[i, j] == Board.Black)
                {
                    this.board.BlackCount++;
                    if (isFrontier)
                        this.board.BlackFrontierCount++;
                    if (this.board.safeDiscs[i, j])
                        this.board.BlackSafeCount++;
                }
                else if (this.board.squares[i, j] == Board.White)
                {
                    this.board.WhiteCount++;
                    if (isFrontier)
                        this.board.WhiteFrontierCount++;
                    if (this.board.safeDiscs[i, j])
                        this.board.WhiteSafeCount++;
                }
                else
                    this.board.EmptyCount++;
            }
        }

    public int GetValidMoveCount(int color)
        {
        int n = 0;

        // Check all board positions.
        int i, j;
        for (i = 0; i < 8; i++)
            for (j = 0; j < 8; j++)
                // If the move is valid for the color, bump the count.
                if (IsValidMove(color, i, j))
                    n++;
        return n;
        }
    
        public void MakeMove(int color, int row, int col)
        {
        // Set the disc on the square.
        board.squares[row, col] = color;
        // Flip any flanked opponents.
        int dr, dc;
        int r, c;
        for (dr = -1; dr <= 1; dr++)
           for (dc = -1; dc <= 1; dc++)
               // Are there any outflanked opponents?
               if (!(dr == 0 && dc == 0) && IsOutflanking(color, row, col, dr, dc))
               {
                   r = row + dr;
                   c = col + dc;
                   // Flip 'em.
                   while (board.squares[r, c] == -color)
                   {
                       board.squares[r, c] = color;
                       r += dr;
                       c += dc;
                   }
               }

        // Update the counts.
        UpdateCounts();
        }
    }
    }
